import pygame
from image import im
from sound import sm
from util import *
from playstate import PlayState

class PlanState:

	def __init__(self,level,path=[]):
		self.level = level
		self.end = False
		#load these
		self.load(level)
		
		
		self.back = im.get("lvl%i.png"%level)
		self.path = [self.atroposPos]
		self.path.extend(path)
		self.state = "idle"
		self.guiButtons = [(pygame.Rect(24,530,40,40),self.buttonPlay),
							(pygame.Rect(90,530,40,40),self.buttonClear),
							(pygame.Rect(156,530,40,40),self.buttonHelp)]
		self.play = False
		self.clickTimeout = 30
		self.help = False
		
	
	def load(self,level):
		try:
			lvl = __import__("level%i"%level,globals())
			self.blocks = lvl.blocks
			self.atroposPos = lvl.atroposPos
		except:
			self.end = True
			self.atroposPos = (0,0)
	
	def buttonHelp(self):
		self.help = True

	def buttonPlay(self):
		self.play = True
	
	def buttonClear(self):
		self.path = [self.atroposPos]
	
	def tick(self):
		if self.help:
			from helpstate import HelpState
			return HelpState(self.level,self.path[1:])
		if self.end:
			from menustate import MenuState
			return MenuState("victory.png")
		if self.play:
			return PlayState(self.level,self.path[1:])
		if not self.clickTimeout:
			self.clickTimeout -= 1
			return self
		pos = pygame.mouse.get_pos()
		buttons = pygame.mouse.get_pressed()
		if self.state == "idle":
			if buttons[0]:
				self.clickTimeout = 15
				#do gui
				for b in self.guiButtons:
					if b[0].collidepoint(pos):
						b[1]()
						return self
				for n in self.path:
					if distSqr(n,pos) < 16:
						self.state = "move"
						self.node = self.path.index(n)
						return self
				else:
					#test for legalness
					legalPoint = True
					if pos[1] >= 500:
						legalPoint = False
					for b in self.blocks:
						if segToRect((pos,self.path[-1]),b):
							legalPoint = False
							break
					if legalPoint:
						self.path.append(pos)
					else:
						pass
						sm.play("beep.ogg")
			elif buttons[2]:
				for i,n in enumerate(self.path):
					if distSqr(n,pos) < 16:
						#check if resulting path is legal
						legal = True
						if i != 0 and i != len(self.path)-1:
							for b in self.blocks:
								seg = (self.path[i-1],self.path[i+1])
								if segToRect(seg,b):
									legal = False
						if legal:
							self.path.remove(n)
						else:
							sm.play("beep.ogg")
						
		elif self.state == "move":
			legalMove = True
			if pos[1] >= 500:
				legalMove = False
			for b in self.blocks:
				if self.node and segToRect((self.path[self.node-1],pos),b):
					legalMove = False
					break
				if self.node != len(self.path)-1 and segToRect((self.path[self.node+1],pos),b):
					legalMove = False
			
			if legalMove:
				self.path[self.node] = pos
			if not buttons[0]:
				self.state ="idle"
				self.clickTimeout = 5
		return self
	
	
	def render(self,screen):
		screen.blit(self.back,(0,0))
		if len(self.path):
			n0 = None
			for node in self.path:
				pygame.draw.circle(screen,0x00ffff,node,4)
				if n0:
					pygame.draw.line(screen,(255,0,0),n0,node)
				n0 = node
		screen.blit(im.get("atropos.png"),(self.atroposPos[0]-10,self.atroposPos[1]-10))
		screen.blit(im.get("planHUD.png"),(0,0))
		screen.blit(im.get("strings%i.png"%self.level),(250,510))

